import {
  Controller,
  Headers,
  InternalServerErrorException,
  Post,
  RawBodyRequest,
  Req,
} from '@nestjs/common';
import { PaymentIntentsService } from 'src/modules/stripe/payment-intents.service';

import { config } from 'dotenv';
import { PreOrdersService } from 'src/modules/orders/pre-orders.service';
import { OrdersService } from 'src/modules/orders/orders.service';
config();

const stripe = require('stripe')(`${process.env.STRIPE_SECRET_KEY}`);
const endpointSecret = `${process.env.STRIPE_WEBHOOK_SECRET_KEY}`;

@Controller('webhook')
export class StripeController {
  constructor(
    private paymentIntentsService: PaymentIntentsService,
    private preOrdersService: PreOrdersService,
    private orderService: OrdersService,
  ) {}

  @Post('')
  async webhook(@Req() req: RawBodyRequest<Request>, @Headers() headers) {
    const sig = headers['stripe-signature'];
    let result: any = true;
    let event;

    try {
      event = stripe.webhooks.constructEvent(req.rawBody, sig, endpointSecret);
    } catch (err) {
      return `Webhook Error: ${err.message}`;
    }

    switch (event.type) {
      case 'checkout.session.completed':
        result = await this.paymentIntentsService.createPaymentIntent(
          event.data.object.payment_intent,
          event.data.object.metadata.preOrderId,
        );
        break;
      case 'payment_intent.succeeded':
        try {
          setTimeout(async () => {
            const paymentIntentSucceeded = event.data.object;
            const paymentIntentObj =
              await this.paymentIntentsService.findOneByPaymentIntentId(
                paymentIntentSucceeded.id,
              );
            if (paymentIntentObj) {
              const preOrder = await this.preOrdersService.getPreOrder(
                paymentIntentObj.preOrderId,
              );
              if (preOrder) {
                this.orderService.create({
                  orderDate: new Date(),
                  total: preOrder.total,
                  userId: preOrder.userId,
                  items: preOrder.items,
                  paymentIntentId: paymentIntentObj.paymentIntentId,
                });
                await this.paymentIntentsService.delete(
                  paymentIntentObj.paymentIntentId,
                );
                await this.preOrdersService.delete(preOrder.id);
              } else {
                throw new InternalServerErrorException('Pre Order not found!');
              }
            } else {
              throw new InternalServerErrorException(
                'Payment intent not found!',
              );
            }
          }, 2000);
        } catch (e) {
          console.log('ERROR PAYMENT SUCCESS: ', e);
        }
        break;
      default:
        console.log(`Unhandled event type ${event.type}`);
    }
    return result;
  }
}
